/* XPLog -- an PPP-Dialer and Phonecost Program
 * Copyright (C) 1998 Stefan Kraus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 

#include "xplog.h"

/* Preference Datas */
extern struct xplog xplogs;

/* German CityPlus Rates */
extern float cityplus;
extern float protcity;

float phcalc(long atime,long btime)
{
  struct tm *aatm;
  struct tm atm,btm;
  float gebuehr=0;

  aatm = localtime(&atime);
  atm.tm_mday=aatm->tm_mday;
  atm.tm_mon=aatm->tm_mon;
  atm.tm_year=aatm->tm_year;
  atm.tm_hour=aatm->tm_hour;
  atm.tm_min=aatm->tm_min;
  atm.tm_sec=aatm->tm_sec;
  aatm = localtime(&btime);
  btm.tm_mday=aatm->tm_mday;
  btm.tm_mon=aatm->tm_mon;
  btm.tm_year=aatm->tm_year;
  btm.tm_hour=aatm->tm_hour;
  btm.tm_min=aatm->tm_min;
  btm.tm_sec=aatm->tm_sec;

  gebuehr=phonecalc(
                    atm.tm_mday,atm.tm_mon+1,atm.tm_year+1900,atm.tm_hour,atm.tm_min,atm.tm_sec, 
                    btm.tm_mday,btm.tm_mon+1,btm.tm_year+1900,btm.tm_hour,btm.tm_min,btm.tm_sec
                    );

  return(gebuehr);

}

float wdaycalc(int weekday,int hourb,int minb,int secb,int houre,int mine,int sece)
{
  int aa;

  /* Rate-Table */
  for (aa=1 ; aa <=70 ; aa++)
    {
      xplogs.xpqt[aa]=float( (xplogs.xpqh[aa]*3600) + (xplogs.xpqm[aa]*60) );
    }

  /* Rate */
  float gebuehr=0;

  /* Timecodes */
  float btime=float((hourb*3600)+(minb*60)+secb); /* Begintime */
  float etime=float((houre*3600)+(mine*60)+sece); /* Endtime */

  int ready=0;
  float bb=0,cc=0;
  while (ready == 0)
    {
      /* Search Rate */
      for (aa=1 ; aa<=10 ; aa++)
        {

          /* Time = 0 - first Rate entry */
          if ( (xplogs.xpqt[aa+ (10*weekday) ]==0) & (aa==1))
            {
              bb=xplogs.xpqr[aa+(10*weekday)];
              cc=xplogs.xpqc[aa+(10*weekday)];
            }

          /* Time < Actual Time - Time Entry > 0 */
          if ((xplogs.xpqt[aa+(10*weekday)]<=btime) && (xplogs.xpqt[aa+(10*weekday)]>0))
            {
              bb=xplogs.xpqr[aa+(10*weekday)];
              cc=xplogs.xpqc[aa+(10*weekday)];
            }
        }

      /* Add One Rate */
      gebuehr=gebuehr+cc;
      /* xpqr = 0 -> xpqr=60 */
      if (bb==0)
        {
          bb=60;
        }
      btime=btime+bb;

      /* Endcontrol */
      if (btime>=etime)
        {
          ready=1;
        }
    }

  return(gebuehr);

}

float phonecalc(int dayb,int monthb,int jearb,int hourb,int minb,int secb,int daye,int monthe,int jeare,int houre,int mine,int sece)
{
  int aa;

  /* Rate-Table */
  for (aa=1 ; aa <=70 ; aa++)
    {
      xplogs.xpqt[aa]=float( (xplogs.xpqh[aa]*3600) + (xplogs.xpqm[aa]*60) );
    }

  /* Rate */
  float gebuehr=0;
  protcity=0;

  /* Actual Date */
  int bday=dayb;
  int bmonth=monthb;
  int bjear=jearb;

  /* Timecodes */
  float btime=float((hourb*3600)+(minb*60)+secb); /* Begintime */
  float etime=float((houre*3600)+(mine*60)+sece); /* Endtime */

  /* Datecodes */
  int bdate=(jearb*10000)+(monthb*100)+dayb;
  int edate=(jeare*10000)+(monthe*100)+daye;

  /* Days in Month */
  int days=monthLen(bmonth,bjear);

  /* Weekday */
  int weekday=getDay(dayb,monthb,jearb);

  int ready=0;
  float bb=0,cc=0;
  while (ready == 0)
    {
      /* Search Rate */
      for (aa=1 ; aa<=10 ; aa++)
        {

          /* Time = 0 - first Rate entry */
          if ( (xplogs.xpqt[aa+ (10*weekday) ]==0) & (aa==1))
            {
              bb=xplogs.xpqr[aa+(10*weekday)];
              cc=xplogs.xpqc[aa+(10*weekday)];
            }

          /* Time < Actual Time - Time Entry > 0 */
          if ((xplogs.xpqt[aa+(10*weekday)]<=btime) & (xplogs.xpqt[aa+(10*weekday)]>0))
            {
              bb=xplogs.xpqr[aa+(10*weekday)];
              cc=xplogs.xpqc[aa+(10*weekday)];
            }
        }

      /* Add One Rate */
      gebuehr=gebuehr+cc;
      /* xpqr = 0 -> xpqr=60 */
      if (bb==0)
        {
          bb=60;
        }
      btime=btime+bb;

      /* CityPlus Range */
      if ((btime>=18000) & (btime<=75600))
        {
          cityplus=cityplus+cc;
          /* for Protocol */
          protcity=protcity+cc;
        }

      /* next day ? */
      if ((btime>=86400) & (bdate<edate))
        {

          /* next month */
          if (bday==days)
            {
              bday=1;
              bmonth++;
              /* next jear */
              if (bmonth>12)
                {
                  bmonth=1;
                  bjear++;
                }
            }
          else
            {
              /* next day */
              if (bday<days)
                {
                  bday++;
                }
            }


          btime=btime-86400;

          bdate=(bjear*10000)+(bmonth*100)+bday;
          weekday=getDay(bday,bmonth,bjear);
          days=monthLen(bmonth,bjear);
        }

      /* Endcontrol */
      if ((bdate>=edate) & (btime>=etime))
        {
          ready=1;
        }
    }

  return(gebuehr);

}
